import type { RegexSequence } from './types';
export interface RegexFlags {
    /** Global search. */
    global?: boolean;
    /** Case-insensitive search. */
    ignoreCase?: boolean;
    /** Allows ^ and $ to match newline characters. */
    multiline?: boolean;
    /** Generate indices for substring matches. */
    hasIndices?: boolean;
    /** Perform a "sticky" search that matches starting at the current position in the target string. */
    sticky?: boolean;
}
/**
 * Generate RegExp object from elements.
 *
 * @param elements Single regex element or array of elements
 * @returns
 */
export declare function buildRegex(sequence: RegexSequence): RegExp;
/**
 * Generate RegExp object from elements with passed flags.
 *
 * @param elements Single regex element or array of elements
 * @param flags RegExp flags object
 * @returns RegExp object
 */
export declare function buildRegex(flags: RegexFlags, sequence: RegexSequence): RegExp;
export declare function _buildRegex(flags: RegexFlags, sequence: RegexSequence): RegExp;
/**
 * Generate regex pattern from elements.
 * @param elements Single regex element or array of elements
 * @returns regex pattern string
 */
export declare function buildPattern(sequence: RegexSequence): string;
//# sourceMappingURL=builders.d.ts.map